#!/usr/bin/env python
# -*- coding:UTF-8 -*-
"""
@Project: rulesHandler
@File: reRuleMatch.py
@Author: Symoon
@Date: 2021/9/13 下午1:51
"""

from functools import wraps
from typing import List

from pydantic import BaseModel

from ValidationRules.ruleHandler import RuleHandler
from ValidationRules.structureFunc import doubleWrap, isNotEmptyString


class StringRules(BaseModel):

    @staticmethod
    @doubleWrap
    def accountRule(func, required: bool = False, label: str = "账号", error_msg: str = ""):
        """
        账号合法校验
        :param error_msg: 错误信息自定义，默认空
        :param label: 标签，错误主题名称
        :param func:
        :param required: 是否必填
        :return:
        """

        @wraps(func)
        def checkAccount(*args, **kwargs):
            """
            校验账号合法
            :param args:
            :param kwargs:
            :return:
            """
            account_list: List[str] = [str(account) for account in args if
                                       type(account) == str and isNotEmptyString(account)]
            if required and not account_list:
                raise ValueError("label {} value {} is not allowed value,check input".format(label, account_list))
            for account_str in account_list:
                RuleHandler.checkAccount(account_str, label=label, error_msg=error_msg)
            return func(*args, **kwargs)

        return checkAccount

    @staticmethod
    @doubleWrap
    def mobileRule(func, required: bool = False, label: str = "手机号", error_msg: str = ""):
        """
        标准手机号校验
        :param error_msg: 错误信息自定义，默认空
        :param label: 标签，错误主题名称
        :param func:被装饰check函数
        comment:支持数字类型手机号。但一般不推荐用数字存储手机号
        :param required: 是否必填
        """

        @wraps(func)
        def checkMobile(*args, **kwargs):
            """
            校验手机号
            :param args:
            :param kwargs:
            :return:
            """
            mobile_str_list: List[str] = [str(mobile) for mobile in args if
                                          type(mobile) == str and isNotEmptyString(mobile) or type(mobile) == int]
            print(mobile_str_list)
            if required and not mobile_str_list:
                raise ValueError("label {} value {} is not allowed value,check input".format(label, mobile_str_list))

            for mobile_str in mobile_str_list:
                RuleHandler.checkMobile(mobile_str, label=label, error_msg=error_msg)
            return func(*args, **kwargs)

        return checkMobile

    @staticmethod
    @doubleWrap
    def keywordCheckRule(func, keyword_list: List[str] = None, startswith: str = None, endswith: str = None,
                         min_length: int = None, max_length: int = None, password_level: int = 0,
                         required: bool = False, label: str = "字符", error_msg: str = ""):
        """
        输入字符验证
        :param error_msg: 错误信息自定义，默认空
        :param label: 标签，错误主题名称
        :param func: 被装饰check函数
        :param keyword_list: 关键字列表
        :param startswith: 以***为开头
        :param endswith: 以***为结尾
        :param min_length: 最小长度
        :param max_length: 最大长度
        :param password_level: 验证等级，默认为0 {3:"三种必须条件",2:"两种必须条件",3:"必须包含字母及数字",0:"不做验证"}
        :param required: 是否必填
        :return:
        """

        @wraps(func)
        def checkKeyword(*args, **kwargs):
            verified_str_list: List[str] = [str(verified_str) for verified_str in args if
                                            type(verified_str) == str and isNotEmptyString(verified_str)]
            if required and not verified_str_list:
                raise ValueError("label {} value {} is not allowed value,check input".format(label, verified_str_list))
            for verified_str in verified_str_list:
                RuleHandler.checkPassword(verified_str, keyword_list=keyword_list, startswith=startswith,
                                          endswith=endswith, min_length=min_length,
                                          max_length=max_length,
                                          password_level=password_level, label=label, error_msg=error_msg)
            return func(*args, *kwargs)

        return checkKeyword

    @staticmethod
    @doubleWrap
    def emailRule(func, required: bool = False, label: str = "邮箱", error_msg: str = None):
        """
        标准邮箱校验
        :param error_msg: 错误信息自定义，默认空
        :param label: 标签，错误主题名称
        :param func:
        :param required: 是否必填
        :return:
        """

        @wraps(func)
        def checkEmail(*args, **kwargs):
            email_str_list = [str(email_str) for email_str in args if
                              type(email_str) == str and isNotEmptyString(email_str)]
            if required and not email_str_list:
                raise ValueError("label {} value {} is not allowed value,check input".format(label, email_str_list))
            for email_str in email_str_list:
                RuleHandler.checkEmail(email_str, label=label, error_msg=error_msg)
            return func(*args, **kwargs)

        return checkEmail

    @staticmethod
    @doubleWrap
    def IDCardRule(func, required: bool = False, label: str = "身份证", error_msg: str = None):
        """
        身份证号校验
        :param error_msg: 错误信息自定义，默认空
        :param label: 标签，错误主题名称
        :param func:
        :param required: 是否必填
        :return:
        """

        @wraps(func)
        def checkIDCard(*args, **kwargs):
            """
            校验身份证号
            :param args:
            :param kwargs:
            :return:
            """
            id_str_list = [str(id_str) for id_str in args if type(id_str) == str and isNotEmptyString(id_str)]
            if required and not id_str_list:
                raise ValueError("label {} value {} is not allowed value,check input".format(label, id_str_list))
            for id_str in id_str_list:
                RuleHandler.checkIDCard(id_str, label=label, error_msg=error_msg)
            return func(*args, **kwargs)

        return checkIDCard



#!/usr/bin/env python
# -*- coding:UTF-8 -*-
"""
@Project: rulesHandler
@File: reRuleMatch.py
@Author: Symoon
@Date: 2021/9/13 下午1:51
"""

from enum import Enum, unique


@unique
class ReRuleMatch(Enum):
    # 标准手机号校验规则
    mobileRule = {"非标准手机号": r"(13\d|14[579]|15[^4\D]|17[^49\D]|18\d)\d{8}"}
    # 满足三种条件  大小写字母数字或者符号
    highPasswordRule = {
        "需要包含大小写字母数字或者符号，满足其中三种条件": r"^(?![a-zA-Z]+$)(?![A-Z0-9]+$)(?![A-Z\\W_!@#$%^&*`~()-+=]+$)(?![a-z0-9]+$)(?![a-z\\W_"
                                    r"!@#$%^&*`~()-+=]+$)(?![0-9\\W_!@#$%^&*`~()-+=]+$)[a-zA-Z0-9\\W_!@#$%^&*`~()-+=]*$"}
    # 满足两种条件字母数字或者符号
    middlePasswordRule = {
        "需要包含字母数字或者符号，满足其中两种条件": r"^(?![a-zA-Z]+$)(?![A-Z\\W_!@#$%^&*`~()-+=]+$)(?![a-z0-9]+$)(?![a-z\\W_"
                                 r"!@#$%^&*`~()-+=]+$)(?![0-9\\W_!@#$%^&*`~()-+=]+$)[a-zA-Z0-9\\W_!@#$%^&*`~()-+=]*$"}
    # 满足数字加字母
    lowPasswordRule = {
        "需要包含字母以及数字": r"^(?=.*\d)(?=.*[a-z]).*$"}

    # 标准邮箱校验规则
    emailRule = {"非标准邮箱": r"^\w+([-+.]\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*$"}

    # 标准身份证号校验规则
    idRule = {"非标准身份证号": r"(^\d{15}$)|(^\d{18}$)|(^\d{17}(\d|X|x)$)"}

    # 校验账号规则
    accountRule = {"非标准账号": r"^[a-zA-Z][a-zA-Z0-9_]{4,15}$"}

    @property
    def getRuleErrorMsg(self):
        """
        根据枚举名称获取规则序号
        :return:
        """
        return list(self.value.keys())[0]

    @property
    def getRuleStr(self):
        """
        根据枚举名称获取规则语句
        :return: 状态说明message
        """
        return list(self.value.values())[0]


if __name__ == "__main__":
    print(ReRuleMatch.mobileRule.mobileRule.getRuleStr)



#!/usr/bin/env python
# -*- coding:UTF-8 -*-
"""
@Project: rulesHandler
@File: ruleHandler.py
@Author: Symoon
@Date: 2021/9/13 下午2:36
"""

import re
from typing import List

from pydantic import BaseModel

from ValidationRules.reRuleMatch import ReRuleMatch


class RuleHandler(BaseModel):
    """
    验证方法类
    """

    @staticmethod
    def checkAccount(account_str: str, label: str = "", error_msg: str = ""):
        """
        校验账号通用方法
        :param error_msg: 错误信息
        :param label: 标签主题
        :param account_str: 账号字符串
        :return:
        """
        res = re.match(ReRuleMatch.accountRule.getRuleStr, account_str)
        if res:
            pass
        else:
            if error_msg:
                raise ValueError(error_msg)
            raise ValueError("【{}】 输入值 【{}】 非标准账号，需要包含英文+数字或纯英文".format(label, account_str))

    @staticmethod
    def checkMobile(mobile: str, label: str = "", error_msg: str = ""):
        """
        校验手机号通用方法
        :param error_msg: 错误信息
        :param label: 标签主题
        :param mobile: 手机号str
        :return:
        """
        if not mobile.isdigit():
            if error_msg:
                raise ValueError(error_msg)
            raise ValueError("【{}】 输入值 【{}】 手机号非纯数字输入".format(label,mobile))
        if len(mobile) != 11:
            if error_msg:
                raise ValueError(error_msg)
            raise ValueError("【{}】 输入值 【{}】 手机号长度不足11位".format(label,mobile))
        reg = re.compile(ReRuleMatch.mobileRule.getRuleStr)
        if not reg.match(mobile):
            if error_msg:
                raise ValueError(error_msg)
            raise ValueError("【{}】 输入值 【{}】 {}".format(label,mobile, ReRuleMatch.mobileRule.getRuleErrorMsg))

    @staticmethod
    def checkKeyWords(verified_str: str, keyword_list: List[str], label: str = "", error_msg: str = ""):
        """
        检查是否包含关键字符
        :param error_msg: 错误信息
        :param label: 标签主题
        :param verified_str: 待验证字符串
        :param keyword_list: 关键字符列表
        :return:
        """
        not_found_keyword_list = [keyword for keyword in keyword_list if not verified_str.find(keyword) >= 0]
        if not_found_keyword_list:
            if error_msg:
                raise ValueError(error_msg)
            raise ValueError("【{}】 输入值【{}】中找不到关键词 {}".format(label,"、".join(not_found_keyword_list), verified_str))

    @staticmethod
    def checkStartSwitch(verified_str: str, startswith: str = None, label: str = "", error_msg: str = ""):
        """
        检查是否以**开头
        :param error_msg: 错误信息
        :param label: 标签主题
        :param verified_str: 待验证字符串
        :param startswith: 开头关键字
        :return:
        """
        if not verified_str.startswith(startswith):
            if error_msg:
                raise ValueError(error_msg)
            raise ValueError("【{}】 输入值【{}】 不是以 {} 为开头".format(label,verified_str, startswith))

    @staticmethod
    def checkEndSwitch(verified_str: str, endswith: str = None, label: str = "", error_msg: str = ""):
        """
        检查是否以**结尾
        :param error_msg: 错误信息
        :param label: 标签主题
        :param verified_str: 待验证字符串
        :param endswith: 结尾关键字
        :return:
        """
        if not verified_str.endswith(endswith):
            if error_msg:
                raise ValueError(error_msg)
            raise ValueError("【{}】 输入值【{}】 不是以 {} 为结尾".format(label,verified_str, endswith))

    @staticmethod
    def checkMinLength(verified_str: str, min_length: int = None, label: str = "", error_msg: str = ""):
        """
        检查最小长度限制
        :param error_msg: 错误信息
        :param label: 标签主题
        :param verified_str: 待验证字符串
        :param min_length: 最小长度参数
        :return:
        """
        if not len(verified_str) >= min_length:
            if error_msg:
                raise ValueError(error_msg)
            raise ValueError("【{}】 输入值【{}】 长度小于 {} 位最小长度限制".format(label,verified_str, min_length))

    @staticmethod
    def checkMaxLength(verified_str: str, max_length: int = None, label: str = "", error_msg: str = ""):
        """
        检查最大长度限制
        :param error_msg: 错误信息
        :param label: 标签主题
        :param verified_str: 待验证字符串
        :param max_length: 最大长度参数
        :return:
        """
        if not len(verified_str) <= max_length:
            if error_msg:
                raise ValueError(error_msg)
            raise ValueError("【{}】 输入值【{}】 长度大于 {} 位最大长度限制".format(label,verified_str, max_length))

    @staticmethod
    def checkPasswordLevel(verified_str: str, password_level: int = 0, label: str = "", error_msg: str = ""):
        """
        验证密码安全等级
        :param error_msg: 错误信息
        :param label: 标签主题
        :param verified_str: 待验证字符串
        :param password_level: 安全等级
        :return:
        """
        level_match = {
            3: ReRuleMatch.highPasswordRule,
            2: ReRuleMatch.middlePasswordRule,
            1: ReRuleMatch.lowPasswordRule
        }
        res = re.search(level_match[password_level].getRuleStr, verified_str)
        if res:
            pass
        else:
            if error_msg:
                raise ValueError(error_msg)
            raise ValueError("【{}】 输入值 【{}】 {}".format(label,verified_str, level_match[password_level].getRuleErrorMsg))

    @classmethod
    def checkPassword(cls, verified_str: str, keyword_list: List[str] = None, startswith: str = None,
                      endswith: str = None,
                      min_length: int = None, max_length: int = None, password_level: int = 0, label: str = "",
                      error_msg: str = ""):

        """
        验证密码通用方法
        :param error_msg: 错误信息
        :param label: 标签主题
        :param verified_str: 待验证字符串
        :param keyword_list: 需要验证的关键字列表
        :param startswith: 以***开头
        :param endswith: 以***结尾
        :param min_length: 最小长度
        :param max_length: 最大长度
        :param password_level: 验证等级，默认为0 {3:"三种必须条件",2:"两种必须条件",3:"必须包含字母及数字",0:"不做验证"}
        :return:
        """
        # 判断是否校验关键字
        if keyword_list:
            cls.checkKeyWords(verified_str, keyword_list,label,error_msg)
        # 判断是否校验开头
        if startswith:
            cls.checkStartSwitch(verified_str, keyword_list,label,error_msg)
        # 判断是否校验结尾
        if endswith:
            cls.checkEndSwitch(verified_str, keyword_list,label,error_msg)
        # 最小长度
        if min_length:
            cls.checkMinLength(verified_str, min_length,label,error_msg)
        # 最大长度
        if max_length:
            cls.checkMaxLength(verified_str, max_length,label,error_msg)
        # 判断鉴定等级
        if password_level != 0:
            cls.checkPasswordLevel(verified_str, password_level,label,error_msg)

    @staticmethod
    def checkEmail(email_str: str, label: str = "", error_msg: str = ""):
        """
        检查是否标准邮箱
        :param error_msg: 错误信息
        :param label: 标签主题
        :param email_str: 邮箱字符串
        :return:
        """
        res = re.match(ReRuleMatch.emailRule.getRuleStr, email_str)
        if res:
            pass
        else:
            if error_msg:
                raise ValueError(error_msg)
            raise ValueError("【{}】 输入值 【{}】 {}".format(label,email_str, ReRuleMatch.emailRule))

    @staticmethod
    def checkIDCard(id_str: str, label: str = "", error_msg: str = ""):
        """
        检查是否标准身份证号
        :param error_msg: 错误信息
        :param label: 标签主题
        :param id_str: 身份证号字符串
        :return:
        """
        res = re.match(ReRuleMatch.idRule.getRuleStr, id_str)
        if res:
            pass
        else:
            if error_msg:
                raise ValueError(error_msg)
            raise ValueError("【{}】 输入值【{}】 {}".format(label,id_str, ReRuleMatch.idRule.getRuleErrorMsg))



#!/usr/bin/env python
# -*- coding:UTF-8 -*-
"""
@Project: rulesHandler
@File: dateTimeHandler.py
@Author: Symoon
@Date: 2021/9/16 上午9:57
"""

from datetime import datetime, timedelta


def _format_time(hh, mm, ss, us, timespec='auto'):
    specs = {
        'hours': '{:02d}',
        'minutes': '{:02d}:{:02d}',
        'seconds': '{:02d}:{:02d}:{:02d}',
        'milliseconds': '{:02d}:{:02d}:{:02d}.{:03d}',
        'microseconds': '{:02d}:{:02d}:{:02d}.{:06d}'
    }

    if timespec == 'auto':
        # Skip trailing microseconds when us==0.
        timespec = 'microseconds' if us else 'seconds'
    elif timespec == 'milliseconds':
        us //= 1000
    try:
        fmt = specs[timespec]
    except KeyError:
        raise ValueError('Unknown timespec value')
    else:
        return fmt.format(hh, mm, ss, us)


def _format_offset(off):
    s = ''
    if off is not None:
        if off.days < 0:
            sign = "-"
            off = -off
        else:
            sign = "+"
        hh, mm = divmod(off, timedelta(hours=1))
        mm, ss = divmod(mm, timedelta(minutes=1))
        s += "%s%02d:%02d" % (sign, hh, mm)
        if ss or ss.microseconds:
            s += ":%02d" % ss.seconds

            if ss.microseconds:
                s += '.%06d' % ss.microseconds
    return s


class FormatDateTime(datetime):

    def __init__(self, year: int, month: int, day: int):
        super().__init__(year, month, day)
        self._microsecond = None
        self._second = None
        self._minute = None
        self._hour = None
        self._day = None
        self._month = None
        self._year = None

    def isoformat(self, sep=' ', timespec='auto'):
        """Return the time formatted according to ISO.

        The full format looks like 'YYYY-MM-DD HH:MM:SS.mmmmmm'.
        By default, the fractional part is omitted if self.microsecond == 0.

        If self.tzinfo is not None, the UTC offset is also attached, giving
        giving a full format of 'YYYY-MM-DD HH:MM:SS.mmmmmm+HH:MM'.

        Optional argument sep specifies the separator between date and
        time, default 'T'.

        The optional argument timespec specifies the number of additional
        terms of the time to include. Valid options are 'auto', 'hours',
        'minutes', 'seconds', 'milliseconds' and 'microseconds'.
        """
        s = ("%04d-%02d-%02d%c" % (self._year, self._month, self._day, sep) +
             _format_time(self._hour, self._minute, self._second,
                          self._microsecond, timespec))

        off = self.utcoffset()
        tz = _format_offset(off)
        if tz:
            s += tz

        return s



#!/usr/bin/env python
# -*- coding:UTF-8 -*-
"""
@Project: rulesHandler
@File: reRuleMatch.py
@Author: Symoon
@Date: 2021/9/13 下午1:51
"""

from functools import wraps


def doubleWrap(func):
    """
    解决装饰器可忽略括号的方案
    """

    @wraps(func)
    def new_dec(*args, **kwargs):
        if len(args) == 1 and len(kwargs) == 0 and callable(args[0]):
            return func(args[0])
        else:
            return lambda real_func: func(real_func, *args, **kwargs)

    return new_dec


def isNotEmptyString(value):
    if value and value != '' and value != "":
        return True
    return False



#!/usr/bin/env python
# -*- coding:UTF-8 -*-
"""
@Project: rulesHandler
@File: setup.py.py
@Author: Symoon
@Date: 2021/9/14 下午5:27
"""

from setuptools import setup

with open("README.md", encoding="utf-8") as f:
    long_description = f.read()
setup(
    name="python-validation-rules",
    packages=["ValidationRules"],
    version="0.1.17",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Symoon",
    author_email="symoon.gao@gmail.com",
    url="https://github.com/Gaoshengyue/validationRules",
    download_url="https://github.com/Gaoshengyue/validationRules/releases/tag/v0.1.17",
    keywords=["python-validation-rules", "validation-rules"],
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent"

    ], install_requires=['pydantic >=1.5.1']
)
